const bcrypt = require("bcryptjs");
const Admin = require("../models/Admin");
const Technician = require("../models/AddTechnician");
const { where } = require("sequelize");

exports.AdminRegister = async (req, res, next) => {
	const defaultImage = "http://localhost:3000/upload_image/default.jpg";
	try {
		const { email, password, fullname, mobilenumber } = req.body; // Extract role from req.body
		console.log(req.body, "BODY");

		// Check if the email already exists in the Admin table
		const existingUser = await Admin.findOne({ where: { email } });
		if (existingUser) {
			return res
				.status(201)
				.json({ status: 0, message: "Email Already Exists" });
		}

		// Hash the password using bcrypt
		const hashedPassword = await bcrypt.hash(password, 10);

		// Create a new admin record in the Admin table
		const admin = await Admin.create({
			email,
			password: hashedPassword,
			fullname,
			mobilenumber,
			profileImages: defaultImage,
		});

		console.log(
			admin,
			"ADMIN>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>"
		);
		return res
			.status(200)
			.json({ status: 1, message: "Admin Register SUCCESS", Data: admin });
	} catch (error) {
		return res
			.status(200)
			.json({ status: 0, message: "Failed", error: error.message });
	}
};

exports.AdminChangepass = async (req, res, next) => {
	const oldPassword = req.body.old_pass;
	const newPassword = req.body.new_pass;
	const confirmPassword = req.body.conf_pass;
	const userId = req.user.id;

	try {
		const admin = await Admin.findByPk(userId);

		if (!admin) {
			console.log("User not found");
			return res.redirect("/editprofile");
		}

		const checkPass = await bcrypt.compare(oldPassword, admin.password);

		if (checkPass) {
			if (newPassword === confirmPassword) {
				const hashedPassword = await bcrypt.hash(newPassword, 10);

				// Update the password using Sequelize's update method
				await admin.update({ password: hashedPassword });

				console.log("Password updated successfully");
				return res.redirect("/editprofile");
			} else {
				console.log("New password and confirmation password do not match");
			}
		} else {
			console.log("Old password is incorrect");
		}

		return res.redirect("/editprofile");
	} catch (error) {
		console.error("Error:", error);
		return res.redirect("/editprofile");
	}
};

exports.dashBoardCounts = async (req, res) => {
	try {
		const userId = req.user.id;

		const admin = await Admin.findOne({
			where: {
				id: userId,
			},
		});

		if (!admin) {
			return res.status(401).json({
				status: 0,
				message: "Un-Authorized Access Denied",
			});
		}

		const techCounts = await Technician.findAndCountAll({});

		return res.status(200).json({
			status: 1,
			message: "Counts Fetch Successfully",
			techCount: techCounts.count,
		});
	} catch (error) {
		console.log("Error :-", error);
		return res.status(500).json({
			status: 0,
			message: "Internal Server Error",
		});
	}
};
