const jwt = require("jsonwebtoken");
const { JWT_SECRET } = require("../config/key");
const nodemailer = require("nodemailer");
const crypto = require("crypto");
const bcrypt = require("bcryptjs");
const Form1 = require("../models/Form1");
const Form2 = require("../models/form2");
const Form3 = require("../models/form3");
const AddTeachnician = require("../models/AddTechnician");
const fs = require("fs-extra");
const puppeteer = require("puppeteer");
const handlebars = require("handlebars");
const path = require("path");
const forgotPassTemp = require("../tamplete/forgotEmail");

exports.UserLogin = async (req, res, next) => {
	try {
		const { email, password, deviceToken, Device_type } = req.body;
		console.log(req.body, "BODY>>>>>>>>>");

		// Assuming the client sends email, password, and deviceToken in the request body

		// Find the user by email
		const loginResult = await AddTeachnician.findOne({ where: { email } });

		if (!loginResult) {
			return res.json({ status: 0, message: "Invalid email" });
		}

		// Check if password is provided in the request
		if (!password) {
			return res.status(200).json({ status: 0, message: "Invalid Password" });
		}

		// Verify the password using bcrypt
		const passwordMatch = await bcrypt.compare(password, loginResult.password);

		if (!passwordMatch) {
			return res.status(200).json({ status: 0, message: "Invalid password" });
		}

		// Create a JWT token to represent the logged-in user
		const token = jwt.sign({ id: loginResult.id }, JWT_SECRET, {
			expiresIn: "1000hr", // Set the token expiration time as needed
		});

		// Update the user's device token and device type in the database
		if (deviceToken || Device_type) {
			loginResult.deviceToken = deviceToken || loginResult.deviceToken;
			loginResult.Device_type = Device_type || loginResult.Device_type;
			await loginResult.save();
		}

		return res.status(200).json({
			status: 1,
			message: "Login Successfully",
			data: {
				id: loginResult.id.toString(),
				email: loginResult.email,
				FullName: loginResult.FullName,
				image: loginResult.image,
				mobilenumber: loginResult.mobilenumber,
				token: token,
			},
		});
	} catch (error) {
		console.error(error);
		res.status(500).json({ message: "Internal Server Error" });
	}
};

exports.UserUpdateProfile = async (req, res, next) => {
	try {
		const { Name, LastName, mobilenumber, email } = req.body;
		const id = req.user.id;

		// Find the user by id
		const user = await AddTeachnician.findByPk(id);

		if (!user) {
			return res.status(404).json({ status: 1, message: "User not found" });
		}

		// Update the user's information
		await user.update({
			Name,
			LastName,
			mobilenumber,
			email,
		});

		// Assuming you have a function to generate a new token (e.g., generateToken)
		const newToken = jwt.sign({ id: req.user.id }, JWT_SECRET, {
			expiresIn: "1d",
		});

		// Prepare the updated user profile response
		const updatedUserProfile = {
			id: user.id.toString(),
			Name: user.Name,
			LastName: user.LastName,
			email: user.email,
			mobilenumber: user.mobilenumber,
			token: newToken,
		};

		res.status(200).json({
			status: 1,
			message: "User updated successfully",
			data: updatedUserProfile,
		});
	} catch (error) {
		console.error(error);
		return res.status(500).json({
			status: 0,
			message: "Failed to update User",
			error: error.message,
		});
	}
};

exports.forgotPass = async (req, res) => {
	try {
		const { email } = req.body;

		if (!email) {
			return res.status(400).json({
				status: 0,
				message: "All Fields Are Required",
			});
		}

		// Find the user by email
		const user = await AddTeachnician.findOne({ where: { email } });

		if (user) {
			// Create a secret key
			const secret = user.id + JWT_SECRET;

			// Generate a token valid for 15 minutes
			const token = jwt.sign({ userId: user.id }, secret, { expiresIn: "15m" });

			// Construct the reset password link
			const link = `http://localhost:3000/reset-password/${user.id}/${token}`;
			const name = user.firstName;

			// Set up email transporter
			const transporter = nodemailer.createTransport({
				host: "smtp.gmail.com",
				port: 465,
				secure: true,
				auth: {
					user: "tryoutscout@gmail.com",
					pass: "xapfekrrmvvghexe",
				},
			});

			// Send the reset password email
			await transporter.sendMail({
				from: "tryoutscout@gmail.com",
				to: user.email,
				subject: "Password Reset Link From Watts",
				text: "This Link is Valid for 15 Minutes Only",
				html: forgotPassTemp({ link, userName: name }),
			});

			return res.status(200).json({
				status: 1,
				message:
					"Reset Password Email Sent Successfully. Please check your email.",
			});
		} else {
			return res.status(400).json({
				status: 0,
				message: "Invalid Email",
			});
		}
	} catch (error) {
		console.error(error);
		return res.status(500).json({
			status: 0,
			message: "Internal Server Error",
		});
	}
};

exports.resetPassword = async (req, res) => {
	const { Password, confirm_password } = req.body;
	const id = req.headers["user-id"];
	const token = req.headers["reset-token"];

	try {
		const user = await AddTeachnician.findByPk(id);

		if (!user) {
			return res.status(404).json({ message: "User not found" });
		}

		const new_secret = user.id + JWT_SECRET;

		try {
			jwt.verify(token, new_secret);
		} catch (err) {
			return res.status(400).json({ message: "Invalid or expired token" });
		}

		if (Password && confirm_password) {
			if (Password !== confirm_password) {
				return res
					.status(400)
					.json({ message: "Password and Confirm Password do not match" });
			}

			const salt = await bcrypt.genSalt(10);
			const hashedPassword = await bcrypt.hash(Password, salt);

			await AddTeachnician.update(
				{ password: hashedPassword },
				{ where: { id: user.id } }
			);

			return res.status(200).json({
				status: 1,
				message: "Password Reset Successfully !!",
			});
		} else {
			return res.status(400).json({ message: "All fields are required" });
		}
	} catch (error) {
		console.error(error);
		return res.status(500).json({
			status: 0,
			message: "Internal Server Error",
		});
	}
};

exports.UserChangePass = async (req, res, next) => {
	const user_id = req.user.id;
	const { old_pass, new_pass, conf_pass } = req.body;

	try {
		const CheckUser = await User.findByPk(user_id);

		if (!CheckUser) {
			return res.status(404).json({ status: 0, message: "User not found" });
		}

		const checkPass = await bcrypt.compare(old_pass, CheckUser.password);

		if (!checkPass) {
			return res
				.status(400)
				.json({ status: 0, message: "Old password does not match" });
		}

		if (new_pass !== conf_pass) {
			return res.status(400).json({
				status: 0,
				message: "New password and confirmation do not match",
			});
		}

		const HashPASS = await bcrypt.hash(new_pass, 10);
		CheckUser.password = HashPASS;
		await CheckUser.save();

		return res.status(200).json({
			status: 1,
			message: "Password updated successfully",
		});
	} catch (error) {
		console.error(error);
		return res
			.status(500)
			.json({ status: 0, message: "Error updating user's password" });
	}
};

//<<<<<<<<<<<<<<<<<<<<<<<< FORM1 CRUD >>>>>>>>>>>>>>>>>
exports.createForm1 = async (req, res) => {
	try {
		const id = req.user.id;

		const {
			customerMail,
			ContractorName,
			LocationName,
			Date,
			Visittype,
			TransformerSerialNumber,
			TransformerLocation,
			TransformerModel,
			TransformerNumberinSite,
			ControlPanelModel,
			Voltage,
			TransformerPower,
			checkItems,
		} = req.body;

		const signature = req.file ? req.file.location : null;

		let parsedCheckItems;
		try {
			parsedCheckItems = JSON.parse(checkItems);
			if (!Array.isArray(parsedCheckItems)) {
				throw new Error("checkItems should be a JSON array.");
			}
		} catch (error) {
			return res.status(400).json({
				status: 0,
				message: "Invalid JSON format for checkItems.",
			});
		}

		const addForm1 = await Form1.create({
			customerMail,
			Technician_id: id,
			ContractorName,
			LocationName,
			Date,
			Visittype,
			TransformerSerialNumber,
			TransformerLocation,
			TransformerModel,
			TransformerNumberinSite,
			ControlPanelModel,
			Voltage,
			TransformerPower,
			checkItems: parsedCheckItems,
			Signature: signature,
		});

		const templatePath = path.join(__dirname, "../htmlPdf/form1.html");
		const templateHtml = await fs.readFile(templatePath, "utf8");
		const template = handlebars.compile(templateHtml);

		const templateData = {
			ContractorName,
			LocationName,
			Date,
			Visittype,
			TransformerSerialNumber,
			TransformerLocation,
			TransformerModel,
			TransformerNumberinSite,
			ControlPanelModel,
			Voltage,
			TransformerPower,
		};

		const htmlToRender = template(templateData);

		const browser = await puppeteer.launch();
		const page = await browser.newPage();

		await page.setContent(htmlToRender, { waitUntil: "networkidle0" });
		await page.emulateMediaType("screen");

		const pdfPath = path.join(__dirname, "Examinationofelactrical.pdf");

		await page.pdf({
			path: pdfPath,
			format: "A4",
			printBackground: true,
		});

		console.log("PDF Generated");
		await browser.close();

		const transporter = nodemailer.createTransport({
			host: "smtp.gmail.com",
			port: 465,
			secure: true,
			auth: {
				user: "tryoutscout@gmail.com",
				pass: "xapfekrrmvvghexe",
			},
		});

		const mailOptions = {
			from: '"Watts" <tryoutscout@gmail.com>',
			to: customerMail,
			subject: `Examinationofelactrical - ${Date}`,
			text: "Please find the attached PDF document for the Examinationofelactrical.",
			attachments: [
				{
					filename: "Examinationofelactrical.pdf",
					path: pdfPath,
				},
			],
		};

		await transporter.sendMail(mailOptions);
		console.log("Email sent to:", customerMail);

		return res.status(200).json({
			status: 1,
			message: "Form One Added Successfully and Email Sent",
			data: addForm1,
		});
	} catch (error) {
		console.log("Error:", error);
		return res.status(500).json({
			status: 0,
			message: "Internal Server Error",
		});
	}
};

//<<<<<<<<<<<<<<<<<<<<<<<< FORM2 CRUD >>>>>>>>>>>>>>>>>
exports.createForm = async (req, res) => {
	try {
		const userId = req.user.id;

		const {
			customerMail,
			date,
			locationName,
			contractorName,
			deviceName,
			deviceSerialNumber,
			suppliedPartSerialNumber,
			installationDate,
			condition,
			deviceAfterInstallation,
			Warranty,
		} = req.body;

		const signature = req.file ? req.file.location : null;

		const form2 = await Form2.create({
			customerMail,
			TechnicianId: userId,
			date,
			locationName,
			contractorName,
			deviceName,
			deviceSerialNumber,
			suppliedPartSerialNumber,
			signature: signature || null,
			installationDate,
			conditionofDeviceafterinstallation: condition,
			deviceAfterInstallation,
			WarrantyForSuppliedPart: Warranty,
		});

		const templatePath = path.join(__dirname, "../htmlPdf/form2.html");
		const templateHtml = await fs.readFile(templatePath, "utf8");
		const template = handlebars.compile(templateHtml);

		const templateData = {
			date,
			locationName,
			contractorName,
			deviceName,
			deviceSerialNumber,
			suppliedPartSerialNumber,
			installationDate,
			condition,
			deviceAfterInstallation,
			Warranty,
		};

		const htmlToRender = template(templateData);

		const browser = await puppeteer.launch();
		const page = await browser.newPage();

		await page.setContent(htmlToRender, { waitUntil: "networkidle0" });
		await page.emulateMediaType("screen");

		const pdfPath = path.join(__dirname, "MashhadSupply.pdf");

		await page.pdf({
			path: pdfPath,
			format: "A4",
			printBackground: true,
		});

		console.log("PDF Generated");
		await browser.close();

		const transporter = nodemailer.createTransport({
			host: "smtp.gmail.com",
			port: 465,
			secure: true,
			auth: {
				user: "tryoutscout@gmail.com",
				pass: "xapfekrrmvvghexe",
			},
		});

		const mailOptions = {
			from: '"Watts" <tryoutscout@gmail.com>',
			to: customerMail,
			subject: `MashhadSupply - ${date}`,
			text: "Please find the attached PDF document for the MashhadSupply.",
			attachments: [
				{
					filename: "MashhadSupply.pdf",
					path: pdfPath,
				},
			],
		};

		await transporter.sendMail(mailOptions);
		console.log("Email sent to:", customerMail);

		res.status(200).json({
			status: 1,
			message: "Form Submitted Successfully and Email Sent",
			data: form2,
		});
	} catch (error) {
		console.log("Error:", error);
		return res.status(500).json({
			status: 0,
			message: "Internal Server Error",
		});
	}
};

exports.updateForm = async (req, res) => {
	try {
		const userId = req.user.id;

		const {
			date,
			locationName,
			contractorName,
			deviceName,
			deviceSerialNumber,
			suppliedPartSerialNumber,
			installationDate,
			condition,
			deviceAfterInstallation,
			Warranty,
		} = req.body;

		const signature = req.file
			? `http://localhost:3000/${req.file.path}`
			: null;

		// Prepare the update object
		const updateData = {
			date,
			locationName,
			contractorName,
			deviceName,
			deviceSerialNumber,
			suppliedPartSerialNumber,
			installationDate,
			conditionofDeviceafterinstallation: condition,
			deviceAfterInstallation,
			WarrantyForSuppliedPart: Warranty,
		};

		// Include signature only if a new file is uploaded
		if (signature) {
			updateData.signature = signature;
		}

		// Update the form with the new data
		const updated = await Form2.update(updateData, {
			where: {
				TechnicianId: userId,
			},
		});

		if (updated) {
			res.status(200).json({
				status: 1,
				message: "Form Detail Updated Successfully",
			});
		} else {
			res.status(404).json({
				status: 0,
				message: "Form not found",
			});
		}
	} catch (error) {
		console.log("Error:", error);
		return res.status(500).json({
			status: 0,
			message: "Internal Server Error",
		});
	}
};

exports.formDetail = async (req, res) => {
	try {
		const userId = req.user.id;
		const { formId } = req.body;

		const verify = await Form2.findOne({
			where: {
				id: formId,
				TechnicianId: userId,
			},
		});

		if (!verify) {
			return res.status(404).json({
				status: 0,
				message:
					"Form Detail Not Found. Invalid Form Id or Unauthorized Access",
			});
		}

		const form2Data = await Form2.findOne({
			where: {
				id: formId,
			},
		});

		res.status(200).json({
			status: 1,
			message: "Form Detail Fetched Successfully",
			data: form2Data,
		});
	} catch (error) {
		console.log("Error:", error);
		return res.status(500).json({
			status: 0,
			message: "Internal Server Error",
		});
	}
};

exports.removeForm = async (req, res) => {
	try {
		const userId = req.user.id;
		const { formId } = req.body;

		const verify = await Form2.findOne({
			where: {
				id: formId,
				TechnicianId: userId,
			},
		});

		if (!verify) {
			return res.status(404).json({
				status: 0,
				message:
					"Form Detail Not Found. Invalid Form Id or Unauthorized Access.",
			});
		}

		await Form2.destroy({
			where: {
				id: formId,
			},
		});

		res.status(200).json({
			status: 1,
			message: "Form Detail Deleted Successfully",
		});
	} catch (error) {
		console.log("Error:", error);
		return res.status(500).json({
			status: 0,
			message: "Internal Server Error",
		});
	}
};

//<<<<<<<<<<<<<<<<<<<<<<<< FORM3 CRUD >>>>>>>>>>>>>>>>>>>>>>
exports.createForm3 = async (req, res) => {
	try {
		const userId = req.user.id;

		const {
			customerMail,
			date,
			locationName,
			visitType,
			contractorName,
			genDataVoltGenerator,
			genDataCYL,
			genDataSN,
			genDataModel,
			genDataKW,
			genDataGENSNSite,
			checkItemOilLevel,
			checkItemOperatingPanel,
			checkItemSmuggling,
			checkItemRadiatorWaterLevel,
			checkItemFuelLevel,
			checkItemMachineFilters,
			checkItemOperatingBatteries,
			checkItemBatteryCharger,
			checkItemCoolingFanBelt,
			checkItemConnectionCables,
			checkItemThestarterofthemachine,
			checkItemvoltageregulator,
			ReadingItemsTime,
			ReadingItemsOilpreacher,
			ReadingItemsTemperature,
			ReadingItemsSpeed,
			ReadingItemsVoltbattery,
			ReadingItemsVoltgenerator,
			ReadingItemsHZ,
			ReadingItemsAM,
			ATSPlatetype,
			ATSAM,
			ATSVoltmines,
			ATSControlscreen,
			ATSControltype,
			ATSCutterstype,
			ATSChargerDC,
			ATSConduction,
		} = req.body;

		const signature = req.file ? req.file.location : null;

		const form3 = await Form3.create({
			TechnicianId: userId,
			customerMail,
			date,
			visitType,
			locationName,
			contractorName,
			genDataVoltGenerator,
			genDataCYL,
			genDataSN,
			genDataModel,
			genDataKW,
			genDataGENSNSite,
			checkItemOilLevel,
			checkItemOperatingPanel,
			checkItemSmuggling,
			checkItemRadiatorWaterLevel,
			checkItemFuelLevel,
			checkItemMachineFilters,
			checkItemOperatingBatteries,
			checkItemBatteryCharger,
			checkItemCoolingFanBelt,
			checkItemConnectionCables,
			checkItemThestarterofthemachine,
			checkItemvoltageregulator,
			ReadingItemsTime,
			ReadingItemsOilpreacher,
			ReadingItemsTemperature,
			ReadingItemsSpeed,
			ReadingItemsVoltbattery,
			ReadingItemsVoltgenerator,
			ReadingItemsHZ,
			ReadingItemsAM,
			ATSPlatetype,
			ATSAM,
			ATSVoltmines,
			ATSControlscreen,
			ATSControltype,
			ATSCutterstype,
			ATSChargerDC,
			ATSConduction,
			signature: signature || null,
		});

		try {
			const templatePath = path.join(__dirname, "../htmlPdf/form3.html");
			const templateHtml = await fs.readFile(templatePath, "utf8");
			const template = handlebars.compile(templateHtml);

			const templateData = {
				Date: date,
				VisitType: visitType,
				LocationName: locationName,
				ContractorName: contractorName,
				VoltGenerator: genDataVoltGenerator,
				CYL: genDataCYL,
				SN: genDataSN,
				Model: genDataModel,
				KW: genDataKW,
				GeneSnSIte: genDataGENSNSite,
				OliLevel: checkItemOilLevel,
				OperatingPanel: checkItemOperatingPanel,
				Smuggling: checkItemSmuggling,
				RadiatorWaterLevel: checkItemRadiatorWaterLevel,
				FuelLevel: checkItemFuelLevel,
				MachineFilter: checkItemMachineFilters,
				OperatingBatteries: checkItemOperatingBatteries,
				BatteryCharger: checkItemBatteryCharger,
				CoolingFanBelt: checkItemCoolingFanBelt,
				ConnectionCables: checkItemConnectionCables,
				Thestarterofthemachine: checkItemThestarterofthemachine,
				voltageregulator: checkItemvoltageregulator,
				Time: ReadingItemsTime,
				Oilpreacher: ReadingItemsOilpreacher,
				ItemsTemperature: ReadingItemsTemperature,
				ItemsSpeed: ReadingItemsSpeed,
				Voltbattery: ReadingItemsVoltbattery,
				Voltgenerator: ReadingItemsVoltgenerator,
				HZ: ReadingItemsHZ,
				AM: ReadingItemsAM,
				Platetype: ATSPlatetype,
				ATSAM: ATSAM,
				Voltmines: ATSVoltmines,
				Controlscreen: ATSControlscreen,
				Controltype: ATSControltype,
				Cutterstype: ATSCutterstype,
				ChargerDC: ATSChargerDC,
				Conduction: ATSConduction,
			};

			const htmlToRender = template(templateData);

			const browser = await puppeteer.launch();
			const page = await browser.newPage();

			await page.setContent(htmlToRender, { waitUntil: "networkidle0" });
			await page.emulateMediaType("screen");

			const pdfPath = path.join(__dirname, "ElectricityGenerator.pdf");

			await page.pdf({
				path: pdfPath,
				format: "A4",
				printBackground: true,
			});

			console.log("PDF Generated");
			await browser.close();

			const transporter = nodemailer.createTransport({
				host: "smtp.gmail.com",
				port: 465,
				secure: true,
				auth: {
					user: "tryoutscout@gmail.com",
					pass: "xapfekrrmvvghexe",
				},
			});

			const mailOptions = {
				from: '"Watts" <tryoutscout@gmail.com>',
				to: customerMail,
				subject: `ElectricityGenerator - ${date}`,
				text: "Please find the attached PDF document for the ElectricityGenerator.",
				attachments: [
					{
						filename: "ElectricityGenerator.pdf",
						path: pdfPath,
					},
				],
			};

			await transporter.sendMail(mailOptions);
			console.log("Email sent to:", customerMail);
		} catch (pdfError) {
			console.log("PDF Generation or Email Sending Error:", pdfError);
			return res.status(500).json({
				status: 0,
				message: "Error generating PDF or sending email. Please try again.",
			});
		}

		return res.status(200).json({
			status: 1,
			message: "Form Submitted Successfully and Email Sent",
			data: form3,
		});
	} catch (error) {
		console.log("Error:", error);
		return res.status(500).json({
			status: 0,
			message: "Internal Server Error",
		});
	}
};

exports.formDetail3 = async (req, res) => {
	try {
		const { formId } = req.body;

		const form3Data = await Form3.findOne({
			where: {
				id: formId,
			},
		});

		if (!form3Data) {
			return res.status(404).json({
				status: 0,
				message: "Form Detail Not Found. Invalid Form Id.",
			});
		}

		res.status(200).json({
			status: 1,
			message: "Form Detail Fetched Successfully",
			data: form3Data,
		});
	} catch (error) {
		console.log("Error:", error);
		return res.status(500).json({
			status: 0,
			message: "Internal Server Error",
		});
	}
};

exports.removeForm3 = async (req, res) => {
	try {
		const { formId } = req.body;

		const form3Data = await Form3.findOne({
			where: {
				id: formId,
			},
		});

		if (!form3Data) {
			return res.status(404).json({
				status: 0,
				message: "Form Detail Not Found. Invalid Form Id.",
			});
		}

		await Form3.destroy({
			where: {
				id: formId,
			},
		});

		res.status(200).json({
			status: 1,
			message: "Form Detail Deleted Successfully",
		});
	} catch (error) {
		console.log("Error:", error);
		return res.status(500).json({
			status: 0,
			message: "Internal Server Error",
		});
	}
};
